﻿// The sole owner of this code is Dundas Data Visualization Inc.
// Copyright (c) 2014 Dundas Data Visualization Inc.
// All rights reserved.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace MyCompany.Sample.TwitterDataProvider
{
	internal static class OAuthHelper
	{
		#region Constants

        /// <summary>
        /// The OAuth Url.
        /// </summary>
		public const string OAuthURl = "https://api.twitter.com/oauth2/token";

		#endregion Constants

		#region Public Methods

		/// <summary>
		/// Gets the authorization headers.
		/// </summary>
		/// <param name="twitterConnectionInfo">The twitter connection information.</param>
		/// <returns>An authorization header as a <see cref="T:System.string" />.</returns>
		public static string GetAuthorizationHeaders(TwitterConnectionInfo twitterConnectionInfo)
		{
			string authHeader =
				string.Format(
					CultureInfo.InvariantCulture,
					"Basic {0}",
					Convert.ToBase64String(
						Encoding.UTF8.GetBytes(
							string.Concat(
								Uri.EscapeDataString(twitterConnectionInfo.ConsumerKey),
								":",
								Uri.EscapeUriString(twitterConnectionInfo.ConsumerSecret)
							)
						)
					)
				);

			string postBody = "grant_type=client_credentials";
			HttpWebRequest authRequest = (HttpWebRequest)WebRequest.Create(OAuthURl);

			authRequest.Headers.Add("Authorization", authHeader);
			authRequest.Method = "POST";
			authRequest.ContentType = "application/x-www-form-urlencoded;charset=UTF-8";
			authRequest.AutomaticDecompression = DecompressionMethods.GZip | DecompressionMethods.Deflate;
			using (Stream stream = authRequest.GetRequestStream())
			{
				byte[] content = ASCIIEncoding.ASCII.GetBytes(postBody);
				stream.Write(content, 0, content.Length);
			}
			authRequest.Headers.Add("Accept-Encoding", "gzip");
			WebResponse authResponse = authRequest.GetResponse();

			using (authResponse)
			{
				using (var reader = new StreamReader(authResponse.GetResponseStream()))
				{
					JavaScriptSerializer javaScriptSerializer = new JavaScriptSerializer();

					Dictionary<string, object> tokenDictionary =
						(Dictionary<string, object>)
							javaScriptSerializer.Deserialize(
								reader.ReadToEnd(),
								typeof(object)
							);

					return string.Format(
						CultureInfo.InvariantCulture,
						"Authorization: {0} {1}",
						tokenDictionary["token_type"],
						tokenDictionary["access_token"]
						);
				}
			}
		}

		#endregion Public Methods
	}
}